import argparse
import json
import logging
import os
import platform
import re
import sys

import requests

__version__ = '1.0.0.dev2'

logger = logging.getLogger(__name__)


def get_replay_directory():
    replay_directory = os.path.expanduser(r'~\Documents\Trackmania\Replays\Autosaves')
    if os.path.isdir(replay_directory):
        return replay_directory
    else:
        logging.warning("Replay directory was not found at expected location: %s.", replay_directory)
        logging.warning("Will try to use the Windows API workaround for finding the directory...")

    import ctypes.wintypes
    CSIDL_PERSONAL = 5  # My Documents
    SHGFP_TYPE_CURRENT = 0  # Get current value (instead of default value)

    buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
    ctypes.windll.shell32.SHGetFolderPathW(None, CSIDL_PERSONAL, None, SHGFP_TYPE_CURRENT, buf)
    replay_directory = buf.value + r"\Trackmania\Replays\Autosaves"
    if os.path.isdir(replay_directory):
        return replay_directory

    return None


def extract_record_from_gbx_file(path):
    best_regexp = re.compile(b'<times best="([0-9]+)"')
    with open(path, 'rb') as fh:
        content = fh.read()
        match = best_regexp.search(content)
        if match:
            best = int(match.group(1).decode('utf-8'))
            return best
    return -1


def main():
    logging.basicConfig(level=logging.INFO)

    parser = argparse.ArgumentParser()
    parser.add_argument('server')
    args = parser.parse_args()

    logger.debug("System: %s" % platform.system())
    if platform.system() != 'Windows':
        logger.error('You are not running Windows. Currently, this tool only supports Windows.')
        sys.exit(1)

    if not args.server.startswith("http://") and not args.server.startswith("https://"):
        logger.error("Invalid server URL. Must start with http:// or https://")
        sys.exit(1)

    replay_directory = get_replay_directory()
    if not replay_directory:
        logging.error("Replay directory could not be found... aborting.")
        sys.exit(1)

    logger.info('Replay directory found at: %s', replay_directory)
    training_autosave_regexp = re.compile(r'^(.*)_(Training - [0-9]+)_PersonalBest_TimeAttack\.Replay\.Gbx$')
    replay_files = []
    for item in os.listdir(replay_directory):
        match = training_autosave_regexp.search(item)
        if match:
            replay_files.append((match.group(1), match.group(2), os.path.join(replay_directory, item)))

    records = []
    for user, track, replay_file in replay_files:
        logger.debug('Processing file "%s"...', replay_file)
        best = extract_record_from_gbx_file(replay_file)
        logger.info("Record for track '%s': %.3fs", track, best / 1000.0)
        records.append({'track': track, 'user': user, 'best': best})

    logger.info("Uploading records...")
    try:
        r = requests.post(args.server,
                          data={
                              'records': json.dumps(records),
                              'client_name': 'upload_tm_records',
                              'client_version': __version__
                          })
        if r.status_code == 200:
            logger.info("SUCCESS!")
        else:
            logger.error("FAILED! Status code: %d. Server response: %s.", r.status_code, r.text)
    except Exception as e:
        logger.error("Could not upload to server. Reason: %s", e)


if __name__ == '__main__':
    main()



import codecs
import os.path
import pathlib
from setuptools import setup, find_packages

def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='upload-tm-records',
    # https://www.python.org/dev/peps/pep-0440/
    version=get_version('src/upload_tm_records.py'),
    description='Upload your trackmania records to a trackmania-records server!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/rsnitsch/trackmania-records',
    author='Robert Nitsch',
    author_email='mail@robertnitsch.de',
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Topic :: Games/Entertainment',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3 :: Only',
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a list of additional keywords, separated
    # by commas, to be used to assist searching for the distribution in a
    # larger catalog.
    keywords='trackmania, gaming',  # Optional

    package_dir={'': 'src'},

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    py_modules=["upload_tm_records"],
    #
    #packages=find_packages(where='src'),  # Required

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'pip install' will check this
    # and refuse to install the project if the version does not match. See
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires='>=3.7, <4',

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests'],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    entry_points={  # Optional
        'console_scripts': [
            'upload-tm-records=upload_tm_records:main',
        ],
    },

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/rsnitsch/trackmania-records/issues',
        'Source': 'https://github.com/rsnitsch/trackmania-records',
    },
)
